//
// ErrorCodes.h
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#ifndef vos_ErrorCodes_h
#define vos_ErrorCodes_h

/*!

  \enum ErrorCodes

  This enum specifies error codes for particular error conditions, and is
  also used to allocate blocks of error numbers to classes.
*/

typedef enum
{
	Err_NoError = 0,

	//
	// Generic errors/failures...
	//
	Err_GenericError = 1,

	Err_OutOfMemory,

	Err_BadParam,

	Err_NotFound, 

	Err_Uninitialised,

	Err_LogicError,

	//
	// Error codes returned by the vos::File class.
	//
	Err_File_Errors = 0x100,

	// 
	// Error codes returned by the vos::Socket class.
	//
	Err_Socket_Errors = 0x200,

	// User-defined error codes should start from here.
	Err_User = 0x100000,

	// Flag to set if the error should be visible to the
	// end user.
	Err_Public = 0x80000000
}
ErrorCodes;

//
// Macro to define error ranges for other libraries/modules.
//
#define ERR_BASE(x) (x+1) << 16

//
// Allocated error code ranges
//

// ERR_BASE(1) => DB library

#endif // !vos_ErrorCodes_h
