//
// Implementation of a list class that isn't a complete bitch to use, unlike
// the MFC version.
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#include "StdHdr.h"

#include "Lists.h"

//
// Enable memory tracking
//
#ifdef _DEBUG
#define new vosDEBUG_NEW
#endif

namespace vos
{
	/******************************************************************************

	>	ListItem::ListItem()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Set members to defaults

	******************************************************************************/

	ListItem::ListItem()
	{
		m_pNext = NULL;
		m_pPrev = NULL;
	}


	/******************************************************************************

	>	ListItem::~ListItem()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Does nothing

	******************************************************************************/

	ListItem::~ListItem()
	{
	}

	/******************************************************************************

	>	ListItem *ListItem::GetNext() const

		Author:		Tim Browse
		Created:	3/2/97
		Returns:	The next item in the list
		Purpose:	return the next item in the list

	******************************************************************************/

	ListItem *ListItem::GetNext() const
	{
		return m_pNext;
	}

	/******************************************************************************

	>	ListItem *ListItem::GetPrev() const

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		The previous item in the list
		Purpose:	returns the previous item in the list

	******************************************************************************/

	ListItem *ListItem::GetPrev() const
	{
		return m_pPrev;
	}


	/******************************************************************************

	>	void ListItem::NotifyListDeleted()

		Author:		Tim Browse
		Created:	3/10/97
		Purpose:	Called when the parent list is being deleted

	******************************************************************************/

	void ListItem::NotifyListDeleted()
	{
		// Default behaviour is to delete this item
		delete this;
	}


	/******************************************************************************

	>	List::List()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	set all member vars to defaults

	******************************************************************************/

	List::List() : m_pHead(NULL), m_pTail(NULL), m_Length(0)
	{
	}

	/******************************************************************************

	>	List::~List()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Makes sure that there is nothing in the list

	******************************************************************************/

	List::~List()
	{
		// Delete all the elements in the list
		DeleteAll();
	}

	/******************************************************************************

	>	ListItem *List::GetHead() const

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Returns the first item in the list

	******************************************************************************/

	ListItem *List::GetHead() const
	{
		return m_pHead;
	}


	/******************************************************************************

	>	ListItem *List::GetTail() const

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	return the last item in the list

	******************************************************************************/

	ListItem *List::GetTail() const
	{
		return m_pTail;
	}

	/******************************************************************************

	>	bool List::IsEmpty()

		Author:		Tim Browse
		Created:	3/2/97
		Returns:	TRUE if the list is empty, FALSE if there is something
					in the list
		Purpose:	To find out if there is anything in the list

	******************************************************************************/

	bool List::IsEmpty()
	{
		return (m_Length == 0);
	}

	/******************************************************************************

	>	uint List::Length()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Find out how many items there are in the list

	******************************************************************************/

	uint List::Length() const
	{
		return m_Length;
	}

	/******************************************************************************

	>	bool List::AddHead(ListItem *pItem)

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		pItem - the list item to add to the head of the list
		Purpose:	Adds a new item to the head of the list

	******************************************************************************/

	bool List::AddHead(ListItem *pItem)
	{
		pItem->m_pNext = m_pHead;
		pItem->m_pPrev = NULL;
		m_pHead = pItem;

		if (m_pHead->m_pNext != NULL)
			m_pHead->m_pNext->m_pPrev = m_pHead;
		if (m_Length == 0)
			m_pTail = m_pHead;

		m_Length++;
		return true;
	}


	/******************************************************************************

	>	bool List::AddTail(ListItem *pItem)

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		pItem - a pointer to the list item to add to the tail of the
					list
		Purpose:	Adds an item to the tail of the list

	******************************************************************************/

	bool List::AddTail(ListItem *pItem)
	{
		pItem->m_pNext = NULL;
		pItem->m_pPrev = m_pTail;
		m_pTail = pItem;

		if (m_pTail->m_pPrev != NULL)
			m_pTail->m_pPrev->m_pNext = m_pTail;
		if (m_Length == 0)
			m_pHead = m_pTail;

		m_Length++;

		return true;
	}


	/******************************************************************************

	>	bool List::InsertBefore(ListItem *pNewItem, ListItem *Pos)

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		pNewItem - A pointer to a new list item to add to the list
					Pos - the item to insert it before
		Returns:	TRUE if it was able to insert the item, FALSE if it failed
		Purpose:	Insert an item before another item

	******************************************************************************/

	bool List::InsertBefore(ListItem *pNewItem, ListItem *Pos)
	{
		// Check for simple case
		if (Pos == m_pHead)
			return AddHead(pNewItem);

		// Otherwise, find the item in question
		ListItem *pItem = m_pHead;
		while ((pItem != NULL) && (pItem != Pos))
			pItem = pItem->m_pNext;

		// Did we find it?
		if (pItem == NULL)
		{
			// No
			return false;
		}

		// Add new item before this one
		pNewItem->m_pNext = Pos;
		pNewItem->m_pPrev = Pos->m_pPrev;
		Pos->m_pPrev = pNewItem;
		pNewItem->m_pPrev->m_pNext = pNewItem;
		m_Length++;

		// Success
		return true;
	}

	/******************************************************************************

	>	bool List::InsertAfter(ListItem *pNewItem, ListItem *Pos)

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		pNewItem - the new list item to insert
					Pos - the item to insert it after
		Returns:	TRUE if it inserted the item, FALSE if it failed
		Purpose:	Inserts a new list item after the Pos item

	******************************************************************************/

	bool List::InsertAfter(ListItem *pNewItem, ListItem *Pos)
	{
		// Check for simple case
		if (Pos == m_pTail)
			return AddTail(pNewItem);

		// Otherwise, find the item in question
		ListItem *pItem = m_pHead;
		while ((pItem != NULL) && (pItem != Pos))
			pItem = pItem->m_pNext;

		// Did we find it?
		if (pItem == NULL)
		{
			// No
			return false;
		}

		// Add new item after this one
		pNewItem->m_pPrev = Pos;
		pNewItem->m_pNext = Pos->m_pNext;
		Pos->m_pNext = pNewItem;
		pNewItem->m_pNext->m_pPrev = pNewItem;
		m_Length++;

		// Success
		return true;
	}


	/******************************************************************************

	>	ListItem *List::Remove(ListItem *pItem)

		Author:		Tim Browse
		Created:	3/2/97
		Inputs:		pItem - the item to remove from the list
		Returns:	A pointer to the item that was removed
		Purpose:	Removes an item from the list. It is not deleted or anything.

	******************************************************************************/

	ListItem *List::Remove(ListItem *pItem)
	{
		// Fix up node pointers
		if (pItem->m_pPrev != NULL)
			pItem->m_pPrev->m_pNext = pItem->m_pNext;

		if (pItem->m_pNext != NULL)
			pItem->m_pNext->m_pPrev = pItem->m_pPrev;

		// Head or Tail may have changed
		if (pItem == m_pHead)
			m_pHead = pItem->m_pNext;
		if (pItem == m_pTail)
			m_pTail = pItem->m_pPrev;

		// NULL item's node pointers
		pItem->m_pPrev = NULL;
		pItem->m_pNext = NULL;

		// One less element in list
		m_Length--;
		ASSERT(m_Length >= 0);

		return pItem;
	}


	/******************************************************************************

	>	void List::RemoveAll()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Removes all the items in the list. None of them are deleted.
					(NB. For ref-counted list items, they are released because they
					are removed from the list.)

	******************************************************************************/

	void List::RemoveAll()
	{
		while (m_pHead != NULL)
			Remove(m_pHead);
	}


	/******************************************************************************

	>	void List::DeleteAll()

		Author:		Tim Browse
		Created:	3/2/97
		Purpose:	Removes all the items in the list and informs each of them
					that the list is being deleted.  Default behaviour is that
					each item then deletes itself.

	******************************************************************************/

	void List::DeleteAll()
	{
		while (m_pHead != NULL)
			Remove(m_pHead)->NotifyListDeleted();
	}



};
