//
// Log.h
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#ifndef vos_Log_h
#define vos_Log_h

#include "String.h"

namespace vos
{
	/**************************************************************************
	
	>	class Log
	
		Author:		Tim Browse
		Date:		15/4/2002
	
		Purpose:	Interface for message logging functionality.
			
					Messages can have a severity and a set of flags.

					The severity controls whether or not a message is logged;
					the Log object can have the severity threshold changed
					via SetSeverity() so that, e.g. it can log only critical 
					errors, or all errors, etc.

					The severity is arbitrary - it's just a number, with no
					meaning other than lower numbers indicate more severe
					messages, e.g. messages of severity 0 are the highest
					priority (and will always be logged).

  		TODO:		Allow logs to have an option to flush every time something 
					is written, to allow debugging of crashes.

	**************************************************************************/
	
	class Log
	{
		public:
			Log();
			virtual ~Log();

		public:
			// Severity - 0 is most severe
			void SetSeverity(uint Severity);

			virtual void LogMsg(const char *szMsg, uint Severity = 0, uint32 Flags = 0) = 0;
			virtual void LogMsg(const String& Msg, uint Severity = 0, uint32 Flags = 0);

		protected:
			// Severity limit - used to cull log messages according to
			// their severity.
			// Initially all messages are logged.
			uint m_Severity;
	};
};

#endif // !vos_Log_h
