//
// MonotonicTime.cpp
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#include "StdHdr.h"

#include "MonotonicTime.h"

#if defined VOS_MSWINDOWS
#include "Win32\Win32_Headers.h"
#endif

//
// Enable memory tracking
//
#ifdef _DEBUG
#define new vosDEBUG_NEW
#endif

namespace vos
{
	/**************************************************************************
	
	>	uint32 GetMonotonicTime()
	
		Author:		Tim Browse
		Date:		15/4/2002
		Returns:	The current time value, measured in milliseconds.
	
		Purpose:	Get the monotonic time value, where time is measured in
					milliseconds.

					This value should only be used for relative comparisons;
					the absolute value is not relevant or defined.
	
	**************************************************************************/
	
	uint32 GetMonotonicTime()
	{
		#ifdef VOS_MSWINDOWS
			return ::GetTickCount();
		#elif (defined VOS_MACOS)
			return ::TickCount();
		#else
			#error GetMonotonicTime() needs porting.
		#endif
	}

	/**************************************************************************
	
	>	void Delay(uint32 Milliseconds)
	
		Author:		Tim Browse
		Date:		15/4/2002
		Inputs:		Milliseconds - the number of milliseconds to wait.
	
		Purpose:	Wait for a period of time. (The function will not
					return until the delay has elapsed.)
	
	**************************************************************************/
	
	void Delay(uint32 Milliseconds)
	{
		uint32 Then = vos::GetMonotonicTime();
		while (vos::GetMonotonicTime() < (Then + Milliseconds))
		{
			// idle
		}
	}

	/**************************************************************************
	
	>	bool PeriodElapsed(uint32 StartTime, uint32 PeriodLength)
	
		Author:		Tim Browse
		Date:		15/4/2002
		Inputs:		StartTime - the start of the period - as obtained via
								GetMonotonicTime().
					PeriodLength - length of the period to check
		Returns:	true => the period has elapsed
					false => otherwise
	
		Purpose:	Check to see if a specific period of time has elapsed.
	
	**************************************************************************/
	
	bool PeriodElapsed(uint32 StartTime, uint32 PeriodLength)
	{
		return ((GetMonotonicTime() - StartTime) > PeriodLength);
	}
};

