//
// ResourceID.h
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#ifndef vos_ResourceID_h
#define vos_ResourceID_h

#include "vos.h"

#include "String.h"

namespace vos
{
#if defined VOS_MSWINDOWS
	typedef LPCTSTR SimpleResourceID;
#elif defined VOS_MACOS
	typedef UInt32 SimpleResourceID;
#else
#error "SimpleResourceID needs to be ported."
#endif

	class Icon;

	class ResourceID
	{
		public:
			// Enum to specify type of a ResourceID object
			enum
			{
				ResType_String,
				ResType_RawData,

				ResType_VOS_Last  // Allows extension of enum
			};

		// Raw constructor is prohibited - use derived classes instead
		private:
			ResourceID();

		protected:
			//
			// Constructor for base class is prohibited for normal use.
			//
			ResourceID(int ResType, int ID = 0);

		public:
			// Copy constructor
			ResourceID(const ResourceID& rhs) 
			{ 
				m_Type = rhs.m_Type;
				m_ID   = rhs.m_ID; 
			}

			// Assignment
			ResourceID& operator=(const ResourceID& rhs)
			{
				m_Type = rhs.m_Type;
				m_ID   = rhs.m_ID;
				return *this;
			}
	
		public:
			// Access functions.
			SimpleResourceID GetID() const { return m_ID; }

			// Returns a ResType_XXX value
			int GetType() const { return m_Type; }

			bool IsNull() const { return m_ID == (SimpleResourceID) 0; }

		private:
			// Data...
			int m_Type;
			SimpleResourceID m_ID;
	};

	class StringResourceID : public ResourceID
	{
	public:
		//
		// Constructor
		//
		StringResourceID() : ResourceID(ResType_String) {}
		StringResourceID(int ID) : ResourceID(ResType_String, ID) {}

		//
		// Comparison
		//
		bool operator==(const StringResourceID& rhs)
		{
			return rhs.GetID() == GetID();
		}

		bool operator==(SimpleResourceID ID)
		{
			return ID == GetID();
		}

		bool operator!=(const StringResourceID& rhs)
		{
			return rhs.GetID() != GetID();
		}

		bool operator!=(SimpleResourceID ID)
		{
			return ID != GetID();
		}

		//
		// Resource loading
		//

		bool Load(vos::String* pString) const;

		vos::String Load() const;
		
		// Convenience function
		void Printf(vos::String* pString, ...);
	};

	class RawDataResourceID : public ResourceID
	{
	public:
		//
		// Constructor
		//
		RawDataResourceID() : ResourceID(ResType_RawData) {}
		RawDataResourceID(int ID) : ResourceID(ResType_RawData, ID) {}
	};
};

#endif // !vos_ResourceID_h


