//
// Socket.h - vOS implementation of socket API
//

#ifndef vos_Socket_h
#define vos_Socket_h

#include "vos\ErrorManager.h"

#include "SocketProtocols.h"

namespace vos
{
	class SocketAddress;

	//
	// Socket error codes
	//
	enum
	{
		// Socket system could not be initialised
		Err_Socket_InitFailed			= Err_Socket_Errors,

		// Socket has not been initialised correctly
		Err_Socket_BadSocket,

		// Socket could not be created
		Err_Socket_CreateFailed,

		// DNS name could not be resolved
		Err_Socket_DNSResolutionFailed,

		// Could not install server on requested port
		Err_Socket_CreateServerFailed,

		// Error occurred when accepting new client connection on a server
		Err_Socket_AcceptClientFailed,

		// Unable to change socket option
		Err_Socket_NonBlockFailed,

		// Unable to connect to server
		Err_Socket_ConnectToServerFailed,

		// Read/write errors
		Err_Socket_WriteFailed,
		Err_Socket_ReadFailed

	};


	/**************************************************************************
	
	>	class Socket
	
		Author:		Tim Browse
		Date:		8/2/2002
	
		Purpose:	Encapsulate a socket object, to make network programming
					simpler.
	
					NB. Currently UNFINISHED!  Do not use!

	**************************************************************************/
	
	class Socket
	{
		public:
			// destructor
			virtual ~Socket() = 0;

			//
			// General API
			//

			// Initialise socket system (call once at app startup)
			static vos::Error Socket::Initialise();

			// Shutdown socket system (call once when app exits)
			static void Socket::Shutdown();

			// Create the socket
			static vos::Error Create(ProtocolFamily Family,
									 ProtocolType	Type,
									 Socket**		ppSocket);

			// Mark the socket as non-blocking
			virtual vos::Error MarkAsNonBlocking() = 0;

			// Binary read/write
			virtual vos::Error Read(void* pData, uint cBytes, uint* pcBytes = NULL) = 0;
			virtual vos::Error Write(const void* pData, uint cBytes, uint* pcBytes = NULL) = 0;

			// Close the socket
			virtual vos::Error Close() = 0;

			//
			// Server API
			//

			// Bind the socket to an address and start listening for
			// connections.
			virtual vos::Error StartServer(uint16 Port, int cBackLog = 5) = 0;

			// Accept a client connection
			virtual vos::Error AcceptClientConnection(Socket** ppNewConnection,
													  SocketAddress* pClientAddress) = 0;


			//
			// Client API
			//

			// Connect the socket to a server
			virtual vos::Error ConnectToServer(SocketAddress ServerAddress) = 0;
	};
}

#endif // !vos_Socket_h
