//
// SocketAddress.h - vOS implementation of socket address type
//

#ifndef vos_SocketAddress_h
#define vos_SocketAddress_h

#include "vos\ErrorManager.h"

#include "SocketProtocols.h"

namespace vos
{
	/**************************************************************************
	
	>	class RawIPAddress
	
		Author:		Tim Browse
		Date:		19/2/2002
	
		Purpose:	Represent a raw 4-byte IP address.
	
	**************************************************************************/
	
	class RawIPAddress
	{
		public:
			// Constructor
			RawIPAddress();

			void Set(uint32 IPAddress) 
			{ 
				m_Address = IPAddress; 
			}

		public:
			union
			{
				// The 4 byte IP address
				byte m_Field[4];

				// The 32-bit IP address
				uint32 m_Address;
			};
	};

	/**************************************************************************
	
	>	class SocketAddress
	
		Author:		Tim Browse
		Date:		19/2/2002
	
		Purpose:	Encapsulate an IP Address/Port combination, and provide
					parsing of textual addresses.
	
	**************************************************************************/
	
	class SocketAddress
	{
		public:
			// Constructors
			SocketAddress();
			SocketAddress(const vos::String& Address, uint16 PortNumber = 0);

			// Access functions
			// (GetAddress() functions can fail if a DNS name is specified
			//  that cannot be resolved)
			vos::Error GetAddress(vos::String* pAddress);
			vos::Error GetAddress(RawIPAddress* pAddress);
			uint	   GetPort() const;

			void SetAddress(const vos::String& Address);
			void SetAddress(const RawIPAddress& Address);
			void SetPort(uint16 PortNumber);

		private:
			// Decode an x.x.x.x form of IP address
			bool DecodeDottedIP(const vos::String& DottedIP,
							    RawIPAddress* pRawIPAddress);

			// Convert a DNS name to a dotted IP address
			vos::Error ResolveDNSName();
			
		private:
			// Address data
			RawIPAddress m_Address;
			uint16 m_Port;

			// DNS name (optional)
			String m_DNSName;
	};
}

#endif // !vos_SocketAddress_h
