//
// Time.cpp
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#include "StdHdr.h"

#include "Time.h"

#include "String.h"

#ifdef VOS_MSWINDOWS

#include "Win32\Win32_Headers.h"

//
// Locale support
//
#include <winnls.h>

#endif



//
// Enable memory tracking
//
#ifdef _DEBUG
#define new vosDEBUG_NEW
#endif

namespace vos
{
	/**************************************************************************
	
	>	Time::Time()
	
		Author:		Tim Browse
		Date:		15/4/2002
	
		Purpose:	Constructor
	
	**************************************************************************/
	
	Time::Time() : m_Time(0)
	{
	}

	/**************************************************************************

	>	Time::Time(int TheYear, MonthName TheMonth, int DayOfMonth,
				   int Hour, int Minutes, int Seconds)


		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		TheYear - year, in full (e.g. 1980, not just 80)
					TheMonth - the month to use
					DayOfMonth - from 1 - 31
					Hour/Minutes/Second - the time of day to use

		Purpose:	Constructor - make a Time object for a specific moment
					in time.

	**************************************************************************/

	Time::Time(int TheYear, MonthName TheMonth, int DayOfMonth,
			   int Hour, int Minutes, int Seconds)
	{
		Set(TheYear, TheMonth, DayOfMonth, Hour, Minutes, Seconds);
	}

	/**************************************************************************

	>	Time::Time(int TheYear, int TheMonth, int DayOfMonth,
				   int Hour, int Minutes, int Seconds)


		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		TheYear - year, in full (e.g. 1980, not just 80)
					TheMonth - the month to use (1-12)
					DayOfMonth - from 1 - 31
					Hour/Minutes/Second - the time of day to use

		Purpose:	Constructor - make a Time object for a specific moment
					in time.

	**************************************************************************/

	Time::Time(int TheYear, int Month, int DayOfMonth,
			   int Hour, int Minutes, int Seconds)
	{
		Set(TheYear, Month, DayOfMonth, Hour, Minutes, Seconds);
	}
	
	/**************************************************************************
	
	>	Time::Time(const Time& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - object to copy
	
		Purpose:	Copy constructor
	
	**************************************************************************/
	
	Time::Time(const Time& rhs)
	{
		// Check for self-copy
		if (this == &rhs)
			return;

		m_Time = rhs.m_Time;
	}

	/**************************************************************************
	
	>	Time::Time(time_t time)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		time - the time_t value to make into a vos Time object.
	
		Purpose:	Constructor using a standard time_t value.
	
	**************************************************************************/
	
	Time::Time(time_t time) : m_Time(time)
	{
	}

	/**************************************************************************
	
	>	Time& Time::operator=(const Time& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the object to assign (copy) to this one
		Returns:	Reference to this object
	
		Purpose:	Assignment operator
	
	**************************************************************************/
	
	Time& Time::operator=(const Time& rhs)
	{
		// Check for self-copy
		if (this == &rhs)
			return *this;

		m_Time = rhs.m_Time;
		return *this;
	}

	/**************************************************************************

	>	void Time::Set(int TheYear, MonthName TheMonth, int DayOfMonth,
					   int Hour, int Minutes, int Seconds)


		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		TheYear - year, in full (e.g. 1980, not just 80)
					TheMonth - the month to use
					DayOfMonth - from 1 - 31
					Hour/Minutes/Second - the time of day to use

		Purpose:	Set this object to hold the date/time value specified by
					the parameters.

	**************************************************************************/

	void Time::Set(int TheYear, MonthName TheMonth, int DayOfMonth,
				   int Hour, int Minutes, int Seconds)
	{
		struct tm TheTime;

		TheTime.tm_year = TheYear - 1900;
		TheTime.tm_mon = (int) TheMonth - (int) January;
		TheTime.tm_mday = DayOfMonth;
		TheTime.tm_hour = Hour;
		TheTime.tm_min = Minutes;
		TheTime.tm_sec = Seconds;
		
		m_Time = mktime(&TheTime);
	}

	/**************************************************************************

	>	void Time::Set(int TheYear, MonthName TheMonth, int DayOfMonth,
					   int Hour, int Minutes, int Seconds)


		Author:		Tim Browse
		Date:		11/9/2002
		Inputs:		TheYear - year, in full (e.g. 1980, not just 80)
					TheMonth - the month to use (1-12)
					DayOfMonth - from 1 - 31
					Hour/Minutes/Second - the time of day to use

		Purpose:	Set this object to hold the date/time value specified by
					the parameters.

	**************************************************************************/

	void Time::Set(int TheYear, int Month, int DayOfMonth,
				   int Hour, int Minutes, int Seconds)
	{
		struct tm TheTime;

		TheTime.tm_year = TheYear - 1900;
		TheTime.tm_mon = Month - 1;
		TheTime.tm_mday = DayOfMonth;
		TheTime.tm_hour = Hour;
		TheTime.tm_min = Minutes;
		TheTime.tm_sec = Seconds;
		
		m_Time = mktime(&TheTime);
	}

	/**************************************************************************
	
	>	void Time::Set(time_t time)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		time - the time_t value to use to set the Time object.
	
		Purpose:	Set this object to hold the date/time value specified by
					the time_t value passed in.
	
	**************************************************************************/
	
	void Time::Set(time_t time)
	{
		m_Time = time;
	}

	/**************************************************************************
	
	>	void Time::SetDate(int TheYear, MonthName TheMonth, int DayOfMonth)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		TheYear - year, in full (e.g. 1980, not just 80)
					TheMonth - the month to use
					DayOfMonth - from 1 - 31
	
		Purpose:	Set the date of a time object - the time will be set
					to 00:00:00
	
	**************************************************************************/
	
	void Time::SetDate(int TheYear, MonthName TheMonth, int DayOfMonth)
	{
		Set(TheYear, TheMonth, DayOfMonth);
	}

	/**************************************************************************
	
	>	void Time::SetTime(int Hour, int Minutes, int Seconds)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		Hour/Minutes/Seconds - the time value to set this object with.
	
		Purpose:	Set the time of a Time object.  The date is set to
					be 1/1/1980
	
	**************************************************************************/
	
	void Time::SetTime(int Hour, int Minutes, int Seconds)
	{
		Set(1980, January, 1, Hour, Minutes, Seconds);
	}

	/**************************************************************************
	
	>	int	Time::GetYear() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The year of this object (in full, e.g. for 1980 it returns
					1980, not just 80)
	
		Purpose:	Get the year component of the Time object.
	
	**************************************************************************/
	
	int	Time::GetYear() const
	{
		TimeInfo Info(*this);
		return Info.m_Year;
	}

	/**************************************************************************
	
	>	MonthName Time::GetMonth() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The name of the month contained in the Time object.
	
		Purpose:	Get the month component of the Time object.
	
	**************************************************************************/
	
	MonthName Time::GetMonth() const
	{
		TimeInfo Info(*this);
		return Info.m_Month;
	}

	/**************************************************************************
	
	>	int	Time::GetDayOfMonth() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The day of the month from this object (1-31)
	
		Purpose:	Find out which day of the month is represented by this
					Time object.
	
	**************************************************************************/
	
	int	Time::GetDayOfMonth() const
	{
		TimeInfo Info(*this);
		return Info.m_DayOfMonth;
	}

	/**************************************************************************
	
	>	DayName Time::GetDayOfWeek() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	Day of the week - e.g. Monday, Tuesday, etc.
	
		Purpose:	Get the day of the week from the date in this Time
					object.
	
	**************************************************************************/
	
	DayName Time::GetDayOfWeek() const
	{
		TimeInfo Info(*this);
		return Info.m_DayOfWeek;
	}

	/**************************************************************************
	
	>	int Time::GetHour() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The hour of the day (range 0-23)
	
		Purpose:	Extract the hour of the day component from this object.
	
	**************************************************************************/
	
	int Time::GetHour() const
	{
		TimeInfo Info(*this);
		return Info.m_Hour;
	}

	/**************************************************************************
	
	>	int Time::GetMinute() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The minutes component (range 0-59)
	
		Purpose:	Extract the minutes component from this object.
	
	**************************************************************************/
	
	int Time::GetMinute() const
	{
		TimeInfo Info(*this);
		return Info.m_Minute;
	}

	/**************************************************************************
	
	>	int Time::GetSeconds() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The seconds component (range 0-59)
	
		Purpose:	Extract the seconds component from this object.
	
	**************************************************************************/
	
	int Time::GetSeconds() const
	{
		TimeInfo Info(*this);
		return Info.m_Seconds;
	}

	/**************************************************************************
	
	>	Time Time::GetCurrentTime()
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The current time/date
	
		Purpose:	Get the current time/date.

					\code

						vos::Time Now = vos::Time::GetCurrentTime();

					\endcode
	
	**************************************************************************/
	
	Time Time::GetCurrentTime()
	{
		Time Now;
		Now.m_Time = time(NULL);
		return Now;
	}


	/**************************************************************************
	
		Time operators
	
		Author:		Tim Browse
		Date:		18/4/2002
	
	**************************************************************************/
	
	bool Time::operator==(const Time& rhs)
	{
		return m_Time == rhs.m_Time;
	}

	bool Time::operator!=(const Time& rhs)
	{
		return m_Time != rhs.m_Time;
	}

	bool Time::operator>(const Time& rhs)
	{
		return m_Time > rhs.m_Time;
	}

	bool Time::operator>=(const Time& rhs)
	{
		return m_Time >= rhs.m_Time;
	}

	bool Time::operator<(const Time& rhs)
	{
		return m_Time < rhs.m_Time;
	}

	bool Time::operator<=(const Time& rhs)
	{
		return m_Time <= rhs.m_Time;
	}

	/**************************************************************************
	
	>	Time Time::operator+(const TimeSpan& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the TimeSpan to add to this Time object.
		Returns:	The resulting Time value
	
		Purpose:	Addition operator for Time/TimeSpan objects.
	
	**************************************************************************/
	
	Time Time::operator+(const TimeSpan& rhs)
	{
		Time NewTime;
		NewTime.m_Time = m_Time + rhs.m_Span;
		return NewTime;
	}

	/**************************************************************************
	
	>	Time Time::operator-(const TimeSpan& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the TimeSpan to subtract from this Time object.
		Returns:	The resulting Time value
	
		Purpose:	Subtraction operator for Time/TimeSpan objects.
	
	**************************************************************************/
	
	Time Time::operator-(const TimeSpan& rhs)
	{
		Time NewTime;
		NewTime.m_Time = m_Time - rhs.m_Span;
		return NewTime;
	}

	/**************************************************************************
	
	>	Time& Time::operator+=(const TimeSpan& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the TimeSpan to add to this object.
		Returns:	Reference to this Time object.
	
		Purpose:	Add and accumulate operator
	
	**************************************************************************/
	
	Time& Time::operator+=(const TimeSpan& rhs)
	{
		m_Time += rhs.m_Span;
		return *this;
	}

	/**************************************************************************
	
	>	Time& Time::operator-=(const TimeSpan& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the TimeSpan to subtract from this object.
		Returns:	Reference to this Time object.
	
		Purpose:	Subtract and accumulate operator
	
	**************************************************************************/
	
	Time& Time::operator-=(const TimeSpan& rhs)
	{
		m_Time -= rhs.m_Span;
		return *this;
	}


	/**************************************************************************
	
	>	TimeSpan Time::operator-(const Time& rhs)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		rhs - the Time object to subtract from this one
		Returns:	TimeSpan representing the difference between the two
					Time objects (in seconds).
	
		Purpose:	Subtract one Time object from another to get a TimeSpan.
	
	**************************************************************************/
	
	TimeSpan Time::operator-(const Time& rhs)
	{
		TimeSpan NewTimeSpan;
		NewTimeSpan.m_Span = m_Time - rhs.m_Time;
		return NewTimeSpan;
	}

	/**************************************************************************
	
	>	String Time::FormatUTCTime(const char* szFormat)
	
		Author:		Tim Browse
		Date:		5/6/2002
		Inputs:		szFormat - the format string to use for the time.
							   This is in the same format as used by the
							   ANSI function strftime() - see standard C
							   library documentation for details.
		Returns:	String representing this time object, formatted as
					specified by the format string provided.
	
		Purpose:	Format a time into a string object, according to the
					format string provided, as a UTC time (i.e. do not
					do any conversions for the local time zone).
	
	**************************************************************************/
	
	String Time::FormatUTCTime(const char* szFormat)
	{
		struct tm *pTime = gmtime(&m_Time);
		const int kBufSize = 256;
		char szTimeBuffer[kBufSize+1];
		strftime(szTimeBuffer, kBufSize, szFormat, pTime);
		return vos::String(szTimeBuffer);
	}

	/**************************************************************************
	
	>	String Time::FormatLocalTime(const char* szFormat)
	
		Author:		Tim Browse
		Date:		5/6/2002
		Inputs:		szFormat - the format string to use for the time.
							   This is in the same format as used by the
							   ANSI function strftime() - see standard C
							   library documentation for details.
		Returns:	String representing this time object, formatted as
					specified by the format string provided.
	
		Purpose:	Format a time into a string object, according to the
					format string provided, using the local time zone to
					correct the time (so that e.g., it is appropriate to show
					the user).
	
	**************************************************************************/
	
	String Time::FormatLocalTime(const char* szFormat)
	{
		struct tm *pTime = localtime(&m_Time);
		const int kBufSize = 256;
		char szTimeBuffer[kBufSize+1];
		strftime(szTimeBuffer, kBufSize, szFormat, pTime);
		return vos::String(szTimeBuffer);
	}
	
	/**************************************************************************
	
	>	String Time::GetTimeSeparator()
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The separator string, e.g. ":"
	
		Purpose:	Obtain a string that can be used to separate components
					of a time value when displaying them.

					For example, this will often be ":", as in "23:11:09".
	
	**************************************************************************/
	
	String Time::GetTimeSeparator()
	{
#ifdef VOS_MSWINDOWS
		// 32 characters should be enough to store the separator!
		const int kBufSize = 32;
		TCHAR szSeparator[kBufSize];

		// Get the locale date string
		::GetLocaleInfo(LOCALE_USER_DEFAULT,
						LOCALE_STIME,
						szSeparator, kBufSize);

		// Convert to string and return
		return String(szSeparator);
#else
		// UNIMPLEMENTED for this platform
		vosERROR();
#endif
	}


#ifdef VOS_MSWINDOWS

	/**************************************************************************
	
	>	void Time::ToSystemTime(SYSTEMTIME *pSysTime) const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Outputs:	pSysTime - this Time object formatted into a SYSTEMTIME object.
	
		Purpose:	Convert a Time object to a Win32 SYSTEMTIME structure, for
					use with Win32 functions.
	
	**************************************************************************/
	
	void Time::ToSystemTime(SYSTEMTIME *pSysTime) const
	{
		TimeInfo Info(*this);

		// Convert time into SYSTEMTIME fields
		pSysTime->wYear = (WORD) Info.m_Year;

		// Convert enum value to 0-based integer
		int Month = ((int) Info.m_Month) - ((int) January);
		pSysTime->wMonth = (WORD) (1 + Month);

		// Convert enum value to 0-based integer
		int DayOfWeek = ((int) Info.m_DayOfWeek) - ((int) Sunday);
		pSysTime->wDayOfWeek = (WORD) DayOfWeek;

		// The rest are simple copies.
		pSysTime->wDay = (WORD) Info.m_DayOfMonth;
		pSysTime->wHour = (WORD) Info.m_Hour;
		pSysTime->wMinute = (WORD) Info.m_Minute;
		pSysTime->wSecond = (WORD) Info.m_Seconds;
		pSysTime->wMilliseconds = 0;
	}

	/**************************************************************************
	
	>	void Time::SetFromSystemTime(const SYSTEMTIME& SysTime)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		SysTime - the Win32 SYSTEMTIME value to convert.
	
		Purpose:	Convert a time value from a Win32 SYSTEMTIME structure, and
					store it in this time object.
	
	**************************************************************************/
	
	void Time::SetFromSystemTime(const SYSTEMTIME& SysTime)
	{
		Set(SysTime.wYear, 
			(MonthName) (January + SysTime.wMonth),
			SysTime.wDay,
			SysTime.wHour,
			SysTime.wMinute,
			SysTime.wSecond);
	}

	//
	// Constants used for converting between FILETIME and time_t values.
	//
	// time_t is the number of seconds elapsed since 
	// midnight (00:00:00), January 1, 1970.
	//
	// FILETIME is the number of 100-nanosecond intervals since
	// January 1, 1601.
	//
	// So kFileTimeFactor is the difference in resolution between the two
	// timing systems.
	//
	// kFileTimeDelta is the difference between the eras of the two 
	// timing systems.
	//
	const LONGLONG kFileTimeDelta = 116444736000000000;
	const LONGLONG kFileTimeFactor = 10000000;

	/**************************************************************************
	
	>	void Time::ToFileTime(FILETIME *pFileTime) const
	
		Author:		Tim Browse
		Date:		2001/10/31
		Outputs:	pFileTime - the converted time.
	
		Purpose:	Convert a vos::Time value to a Win32 FILETIME format
					time structure.
	
	**************************************************************************/
	
	void Time::ToFileTime(FILETIME *pFileTime) const
	{
		// Convert accordingly (code taken from MS sample).
		LONGLONG ll = Int32x32To64(m_Time, kFileTimeFactor) + kFileTimeDelta;
	    pFileTime->dwLowDateTime = (DWORD) ll;
	    pFileTime->dwHighDateTime = ll >> 32;

	}

	/**************************************************************************
	
	>	void Time::SetFromFileTime(const FILETIME& FileTime)
	
		Author:		Tim Browse
		Date:		2001/10/31
		Inputs:		FileTime - the filetime object to convert from.
	
		Purpose:	Set the time object from a Win32 FILETIME object.
	
	**************************************************************************/
	
	void Time::SetFromFileTime(const FILETIME& FileTime)
	{
		// Reverse of the function above.
		LONGLONG Hi = (LONGLONG) FileTime.dwHighDateTime;
		LONGLONG Lo = (LONGLONG) FileTime.dwLowDateTime;

		LONGLONG ll = (Hi << 32) | Lo;
		
		m_Time = (time_t) ((ll - kFileTimeDelta) / kFileTimeFactor);
	}

#endif

	/**************************************************************************
	
	>	TimeInfo::TimeInfo(const Time& TheTime)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		TheTime - the Time object to decompose.
	
		Purpose:	Construct a TimeInfo object from the specified Time object.

					This is the easiest (and most efficient) way to extract 
					multiple components from this Time object.
	
	**************************************************************************/
	
	TimeInfo::TimeInfo(const Time& TheTime)
	{
		// Convert to time components
		struct tm *pTime = localtime(&TheTime.m_Time);

		// Add 1900 - yet another reason to write my own time implementation!
		m_Year = 1900 + pTime->tm_year;

		// Month = 0-11, January = 0
		m_Month = (MonthName) (January + pTime->tm_mon);

		// Day numbering starts from 1
		m_DayOfMonth = pTime->tm_mday;

		// Day is 0-6, Sunday = 0
		m_DayOfWeek = (DayName) (Sunday + pTime->tm_wday);

		// Hours, Mins, Secs fairly standard
		m_Hour = pTime->tm_hour;
		m_Minute = pTime->tm_min;
		m_Seconds = pTime->tm_sec;
	}

	/**************************************************************************
	
	>	TimeSpan::TimeSpan()
	
		Author:		Tim Browse
		Date:		18/4/2002
	
		Purpose:	Constructor (object represents a span of 0 seconds)
	
	**************************************************************************/
	
	TimeSpan::TimeSpan() : m_Span(0)
	{
	}

	/**************************************************************************
	
	>	void TimeSpan::SetWeeks(int cWeeks)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		cWeeks - the number of weeks to store in this TimeSpan.
	
		Purpose:	Set this TimeSpan object to represent a specific number
					of weeks.
	
	**************************************************************************/
	
	void TimeSpan::SetWeeks(int cWeeks)
	{
		m_Span = cWeeks * 7 * 24 * 60 * 60;
	}

	/**************************************************************************
	
	>	void TimeSpan::SetDays(int cDays)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		cDays - the number of days to store in this TimeSpan.
	
		Purpose:	Set this TimeSpan object to represent a specific number
					of days.
	
	**************************************************************************/
	
	void TimeSpan::SetDays(int cDays)
	{
		m_Span = cDays * 24 * 60 * 60;
	}

	/**************************************************************************
	
	>	void TimeSpan::SetHours(int cHours)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		cHours - the number of hours to store in this TimeSpan.
	
		Purpose:	Set this TimeSpan object to represent a specific number
					of hours.
	
	**************************************************************************/
	
	void TimeSpan::SetHours(int cHours)
	{
		m_Span = cHours * 60 * 60;
	}

	/**************************************************************************
	
	>	void TimeSpan::SetMinutes(int cMinutes)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		cMinutes - the number of minutes to store in this TimeSpan.
	
		Purpose:	Set this TimeSpan object to represent a specific number
					of minutes.
	
	**************************************************************************/
	
	void TimeSpan::SetMinutes(int cMinutes)
	{
		m_Span = cMinutes * 60;
	}

	/**************************************************************************
	
	>	void TimeSpan::SetSeconds(int cSeconds)
	
		Author:		Tim Browse
		Date:		18/4/2002
		Inputs:		cSeconds - the number of seconds to store in this TimeSpan.
	
		Purpose:	Set this TimeSpan object to represent a specific number
					of seconds.
	
	**************************************************************************/
	
	void TimeSpan::SetSeconds(int cSeconds)
	{
		m_Span = cSeconds;
	}

	/**************************************************************************
	
	>	int TimeSpan::GetYears() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of years in this object
	
		Purpose:	Get the number of years represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetYears() const
	{
		return m_Span / (60 * 60 * 24 * 365);
	}

	/**************************************************************************
	
	>	int TimeSpan::GetWeeks() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of weeks in this object
	
		Purpose:	Get the number of weeks represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetWeeks() const
	{
		return m_Span / (60 * 60 * 24 * 7);
	}

	/**************************************************************************
	
	>	int TimeSpan::GetDays() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of days in this object
	
		Purpose:	Get the number of days represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetDays() const
	{
		return m_Span / (60 * 60 * 24);
	}

	/**************************************************************************
	
	>	int TimeSpan::GetHours() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of hours in this object
	
		Purpose:	Get the number of hours represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetHours() const
	{
		return m_Span / (60 * 60);
	}

	/**************************************************************************
	
	>	int TimeSpan::GetMinutes() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of minutes in this object
	
		Purpose:	Get the number of minutes represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetMinutes() const
	{
		return m_Span / 60;
	}

	/**************************************************************************
	
	>	int TimeSpan::GetSeconds() const
	
		Author:		Tim Browse
		Date:		18/4/2002
		Returns:	The number of seconds in this object
	
		Purpose:	Get the number of seconds represented by this TimeSpan object.
	
	**************************************************************************/
	
	int TimeSpan::GetSeconds() const
	{
		return m_Span;
	}

	/**************************************************************************
	
		TimeSpan arithmetic
	
		Author:		Tim Browse
		Date:		18/4/2002
	
	**************************************************************************/

	TimeSpan TimeSpan::operator+(const TimeSpan& rhs)
	{
		TimeSpan NewTimeSpan;
		NewTimeSpan.m_Span = m_Span + rhs.m_Span;
		return NewTimeSpan;
	}

	TimeSpan TimeSpan::operator-(const TimeSpan& rhs)
	{
		TimeSpan NewTimeSpan;
		NewTimeSpan.m_Span = m_Span - rhs.m_Span;
		return NewTimeSpan;
	}

	TimeSpan& TimeSpan::operator+=(const TimeSpan& rhs)
	{
		m_Span += rhs.m_Span;
		return *this;
	}

	TimeSpan& TimeSpan::operator-=(const TimeSpan& rhs)
	{
		m_Span -= rhs.m_Span;
		return *this;
	}

};

