//
// VFS_Raw.h
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#ifndef vos_VFS_Raw_h
#define vos_VFS_Raw_h

#include "vos.h"

#include "VFS.h"

namespace vos
{
	/**************************************************************************

		class VFS_Raw : public VFS

		Author:		Tim Browse

		Date:		13/2/00

		Purpose:	Implements the basic 'pass-through' VFS class.

					This VFS class allows a standard OS folder to be opened as
					a VFS hierarchy.  This is therefore a very simple class,
					but a very important one, as it maintains the abstraction
					of a VFS, by allowing the same code to be used, regardless
					of whether it's a VFS or a RFS (Real Filing System :-)).

	**************************************************************************/

	class VFS_Raw : public VFS
	{
		public:
			VFS_Raw(const FileSpec& RootFS);

		public:
			// Open a file within the VFS
			virtual Error OpenFile(const FileSpec& RelativeFS, 
								   FileMode Mode,
								   File** ppFile);

			// Enumerate the catalogue of a VFS
			virtual FileMatches* EnumFiles(const FileSpec& RelativeFS);

		private:
			// Location of the VFS root
			FileSpec m_RootFS;
	};
};

#endif // !vos_VFS_Raw_h
