//
// ZipCatalogueReader.h
//

#ifndef util_ZipCatalogueReader_h
#define util_ZipCatalogueReader_h

#include "vos.h"

namespace vos
{
	class ZipFileInfo;
	class RawZipFileHeader;

	
	/**************************************************************************

	>	class ZipCatalogueReader

		Author:		Tim Browse

		Date:		07/06/2001

		Purpose:	Utility class to use when reading in a ZIP catalogue from
					a ZIP file.

					This class allows the important information to be stored
					from the catalogue, allowing searches for files via filename,
					and access to where the files are in the ZIP file.


					See ZipReader class for an example of how to use this class.

					Better still, just actually use the ZipReader class!

	**************************************************************************/

	class ZipCatalogueReader
	{
		public:
			ZipCatalogueReader();
			~ZipCatalogueReader();

			//
			// Building the catalogue
			//

			// Allocate buffer for the catalogue
			//
			// (NB. you must do this before adding any files!)
			bool AllocateCatalogue(int cFiles);

			// Add the filename to the catalogue.
			void AddFile(char *Filename, 
						 const RawZipFileHeader& FileHeader);

			//
			// Using the catalogue
			//

			// Find out how many files are in the catalogue.
			uint GetNumFiles() const { return m_cMaxFiles; }

			// Getting info for a particular catalogue entry.
			void GetFilename(uint iFile, vos::FileSpec* pFileSpec);

			bool GetFile(uint iFile,
						 uint *pFileOffset,
						 uint *pFileLength,
						 uint32 *pCRC32 = NULL);


			// Finding a named file in the catalogue (and accessing its info).
			bool FindFile(const vos::FileSpec& FS, uint* piFile);

			bool FindFile(const vos::FileSpec& FS,
						  uint *pFileOffset,
						  uint *pFileLength,
						  uint32 *pCRC32 = NULL);

		private:
			//
			// The data for the catalogue
			//

			// Maximum number of files allowed in this catalogue.
			int m_cMaxFiles;

			// Number of files added to this catalogue
			int m_cFiles;

			// The file info entries
			ZipFileInfo* m_pFiles;
	};
}

#endif // !util_ZipCatalogueReader_h

