//
// Win32RegistryKey.h
//

//
// This software is protected by the GNU Lesser General Public License,
// also known as the 'GNU library License', or the LGPL.
//
// See the file COPYING.txt for details.
//

#ifndef vos__Win32RegistryKey_h
#define vos__Win32RegistryKey_h

#include "..\vOS.h"
#include "..\String.h"

#include "windows.h"

namespace vos
{
	/**************************************************************************
	
	>	class Win32RegistryKey
	
		Author:		Tim Browse
		Date:		18/4/2002
	
		Purpose:	Provide a class to encapsulate a Win32 registry key, and
					provide functions for performing common operations on the
					key.
	
	**************************************************************************/
	
	class Win32RegistryKey
	{
		public:
			// Constructors
			Win32RegistryKey();

			Win32RegistryKey(HKEY hParentKey,
							 const String& KeyName);
			Win32RegistryKey(const Win32RegistryKey& ParentKey,
							 const String& KeyName);

			Win32RegistryKey(const Win32RegistryKey& rhs);

			// Destructor
			~Win32RegistryKey();

			// Assignment operator.
			const Win32RegistryKey& operator=(const Win32RegistryKey& rhs);

			bool Open(HKEY hParentKey, const String& KeyName);
			bool Open(const Win32RegistryKey& ParentKey, const String& KeyName);

			bool Create(HKEY hParentKey, const String& KeyName);
			bool Create(const Win32RegistryKey& ParentKey, const String& KeyName);

			bool IsValid() const;

			// Get the HKEY attached to this object
			HKEY GetHKEY() const;

			// Get and detach the HKEY from this object - the caller must dispose of the HKEY.
			HKEY DetachHKEY();

		private:
			void CleanUp();

		private:
			// They key for this object
			HKEY m_hKey;

			// The parent key and keyname for this key
			HKEY m_hParentKey;
			String m_KeyName;
	};
};

#endif // !vos__Win32RegistryKey_h
